#' Data Upload UI Component
#' 
#' This function creates a set of UI elements for data uploading in a Shiny app
#'
#' @param id A unique identifier for the UI module. This is necessary for distinguishing 
#'           this module's input from others in the same Shiny application.
#'
#' @return
#' @export
#'
#' @examples
#' DataUploadUI("DataUploadModuleUI")
DataUploadUI <- function(id) {
  ns <- NS(id)
  tagList(
    fluidRow(
      column(4,
             h2('Upload Data for Analysis'),
             
             # File options for analysis
             radioButtons(
               ns("fileoption"), 
               "Select type of data upload", 
               choices = c("Processed Data from CogentAP", "Example data"), 
               selected = "Processed Data from CogentAP"
             ),
             
             bsPopover(ns("fileoption"), "File upload options", 
                       "CogentDS accepts Processed Data from CogentAP in .rds format. Please choose Example data to try all app functions in case you do not have your data in the required format.", 
                       placement = "right", trigger = "hover", options = list(container = "body")),
             
             # Conditional panel for "Example data"
             conditionalPanel(
               condition = sprintf("input['%s'] == 'Example data'", ns("fileoption")),
               actionButton(ns("uploadData"), "Upload Data", style = 'color:#fff; background-color:#1c82e0')
             ),
             
             # Conditional panel for "Processed Data from CogentAP"
             conditionalPanel(
               condition = sprintf("input['%s'] == 'Processed Data from CogentAP'", ns("fileoption")),
               fileInput(ns("file1"), "Select data for CNV analysis", accept = ".rds"),
               actionButton(ns("uploadData"), "Upload Data", style = 'color:#fff; background-color:#1c82e0')
             )
      )
    )
  )
}
