#' CN Profile Plot UI Component
#'
#' Generates UI elements for a module dedicated for visualizing CN profile plot
#' 
#' @param id A unique identifier for the UI module. This is necessary for distinguishing 
#'           this module's input from others in the same Shiny application.
#'
#' @return
#' @export
#'
#' @examples
#' ccnplotUI("ccnplotModuleUI")
ccnplotUI <- function(id) {
  ns <- NS(id)
  tagList(
    h2('CN Profile'),
    fluidRow(
      column(4,
             # Select barcode for visualization
             selectInput(ns("barcode"), "Select Barcode:", choices = NULL, multiple = T),
             bsPopover(ns("barcode"), "Select barcode/barcodes to plot", 
                       "Select upto three barcodes for CN Profile visualization.", 
                       placement = "right", trigger = "hover", options = list(container = "body")),
             
             
             # Button to plot
             actionButton(ns("plot"), "Generate plot", style='color:#fff; background-color:#1c82e0'),
             # Select file type for downloading
             selectInput(ns("plot_file_type"), "File Type Selection:", choices = list("PNG" = ".png", "PDF" = ".pdf", "SVG" = ".svg", "JPEG" = ".jpeg"), selected = ".png"),
             # Download plot button
             downloadButton(ns('download_plot'), 'Save plot', style='color:#fff; background-color:#1c82e0'),
      )
    ),
    fluidRow(
      uiOutput(ns("plotUI"))
    )
  )
}