#' UI logic for Lasso selection
#' 
#' Generates UI elements for the UMAP lasso selection module
#' 
#' @param id A unique identifier for the UI module. This is necessary for distinguishing 
#'           this module's input from others in the same Shiny application.
#'
#' @return
#' @export
#'
#' @examples
#' lassoUI("lassoUIModule")
lassoUI <- function(id) {
  ns <- NS(id)
  tagList(
    h2('Custom lasso selection'),
    fluidRow(
      column(4,
             # Input for outlier detection threshold
             numericInput(ns("niqrs"), "Outlier detection threshold", value = 3),
             bsPopover(ns("niqrs"), "Outlier Detection Threshold", 
                       "Adjust the threshold to filter cells based on Gini scores.", 
                       placement = "right", trigger = "hover", options = list(container = "body")),
             
             # Dynamic input for filtering based on sample, ploidy, and barcode
             selectInput(ns("sample"), "Select Sample:", choices = NULL, multiple = TRUE),
             bsPopover(ns("sample"), "Select Sample", 
                       "Filter by samples.", 
                       placement = "right", trigger = "hover", options = list(container = "body")),
             
             
             sliderInput(ns("ploidy"), "Select Ploidy Range:", 
                         min = 0, max = 1, 
                         value = c(0, 1), step = 0.1),
             bsPopover(ns("ploidy"), "Ploidy Range", 
                       "Set the range of ploidy values to filter the data.", 
                       placement = "right", trigger = "hover", options = list(container = "body")),
             
             selectizeInput(ns("barcode"), "Select Barcode:", choices = NULL, multiple = TRUE),
             bsPopover(ns("barcode"), "Select Barcode", 
                       "Filter data by specific barcodes.", 
                       placement = "right", trigger = "hover", options = list(container = "body")),
             
             
             selectInput(ns("color_by"), "Color by", choices = c("sample", "cluster"), selected = "sample"),
             bsPopover(ns("color_by"), "Color by", 
                       "Choose whether to color UMAP by samples or clusters.", 
                       placement = "right", trigger = "hover", options = list(container = "body")),
             
             
             uiOutput(ns("downloadUI")),

      ),
      
      column(8,
             shinycssloaders::withSpinner(plotlyOutput(ns("UMAP_plot"), width = "600px", height = "500px"))
      )
    )
  )
}