#' Barcode Rank Plot UI Component
#'
#' This function allows user to upload a barcde counts estmated from demux
#'
#' @param id A unique identifier for the UI module. This is necessary for distinguishing 
#'           this module's input from others in the same Shiny application.
#'
#' @return A UI element comprising downloadable knee plot. The element is wrapped in a `tagList` for easy 
#'         integration into the app's UI.
#' @export
#'
#' @examples
#' In the UI definition of a Shiny app
#' barcoderankUI("barcoderankModule")
#' 
#' @note The object the user uploads needs to have a processed seurat v5 structure

barcoderankUI <- function(id) {
  ns <- NS(id)
  
  tagList(
    fluidRow(
      column(3,
        h2("Barcode Rank Plot"),
        fileInput(ns("barcode_counts"), "Upload your Barcode Counts from Demux in .csv", accept = c(".csv")),
        actionButton(ns("generateKneeplot"), label = "Generate Barcode Rank Plot", class = "btn-primary", icon = icon("upload"), style='color:#fff; background-color:#1c82e0'),
        conditionalPanel(
          condition = "input.generateKneeplot > 0",
          downloadButton(ns('downloadRanks'), 'Download Barcode Ranks', style='color:#fff; background-color:#1c82e0'), ns=NS(id))
      ),
      column(9,
        conditionalPanel(
          condition = "input.generateKneeplot > 0",
          selectInput(ns("kneeplot"), "File Type Selection:", choices = list("PNG" = ".png", "PDF" = ".pdf", "SVG" = ".svg", "JPEG" = ".jpeg"), selected = ".png"),
          downloadButton(ns('downloadPlot_knee'), 'Save plot', style='color:#fff; background-color:#1c82e0'),
          shinycssloaders::withSpinner(plotlyOutput(ns("kneeplot_static"), width = "700", height = "700"), size = 1), ns=NS(id))
      )
    )
  )
}
