downloadSeurat_v5_Server <- function(id, seuratObj) {
  
  moduleServer(id, function(input, output, session) {
    ns <- session$ns
    output$scRNAObjectDownload <- downloadHandler(
      filename = function() {
      paste0("CogentDS_Analysis_Processed_Seurat_v5_Object", "-", Sys.Date(), ".rds")
      },
      content = function(file) {
        withProgress(message = 'Preparing Data for Download...', value = 0, {
          incProgress(0.5, detail = "Downloading..Please Wait...")
          tryCatch({
            sc_object = seuratObj()
            parameter_list = sc_object@assays$user_parameter_tracker
            check_original_presence = grep('Original Data: ', parameter_list, value = T)
            check_isoform_presence = grep('Isoform Data: ', parameter_list, value = T)
            check_biotype_presence = grep('RNABiotype Data: ', parameter_list, value = T)
            check_varfeature_presence = grep('VariableFeatures Data: ', parameter_list, value = T)
            if(length(check_original_presence) > 0) {
              original_file_split = strsplit(check_original_presence, split = "Original Data: ", fixed = T)
              if(length(check_isoform_presence) > 0) {
                isoform_file_split = strsplit(check_isoform_presence, split = "Isoform Data: ", fixed = T)
                unlink(isoform_file_split[[1]][2])
              }
              incProgress(0.3, detail = "Cleaning up!")
              if(length(check_biotype_presence) > 0) {
                original_file_split_biotype = strsplit(check_biotype_presence, split = "RNABiotype Data: ", fixed = T)
                unlink(original_file_split_biotype[[1]][2])
              }
              if(length(check_varfeature_presence) > 0) {
                original_file_split_varfeatures = strsplit(check_varfeature_presence, split = "VariableFeatures Data: ", fixed = T)
                unlink(original_file_split_varfeatures[[1]][2])
              }
              unlink(original_file_split[[1]][2])
              sc_object@assays$user_parameter_tracker = NULL
              saveRDS(sc_object, file = file)  
              incProgress(0.2, detail = "Downloaded")
            } else {
              sc_object@assays$user_parameter_tracker = NULL
              saveRDS(sc_object, file = file)
              incProgress(0.5, detail = "Downloaded")
            }
            shinyjs::disable("scRNAObjectDownload")
            shinyalert('Processed Seurat v5 Object Downloaded', "Thank you! CogentDS Team", type = "success", confirmButtonCol = "#1c82e0")
            }, error = function(e) {
            shinyalert('Error in downloading the processed Seurat v5 Object', 'Please check your input parameters', type = "error", confirmButtonCol = "#1c82e0")
          })
        })
      }
    )
  })
}