generateReport_Server <- function(id, gene_seuratObj_processed, isoform_seuratObj_processed) {
  
  moduleServer(id, function(input, output, session) {
    ns <- session$ns
    
    observeEvent(input$runMarkers, {
      markers_genes = NULL
      markers_isoforms = NULL
      req(gene_seuratObj_processed(), input$runMarkers)
      withProgress(message = 'Generating Markers for Report...', value = 0, {
        incProgress(0.2, detail = "Generating elements ...")
        tryCatch({
          if(input$runMarkers == "TRUE") {
            processed_gene_data = gene_seuratObj_processed()
            parameter_list <- processed_gene_data@assays$user_parameter_tracker
            incProgress(0.4, detail = "Obtaining markers ...")
            if(!is.null(processed_gene_data)) {
              Idents(processed_gene_data) = "seurat_clusters"
              if(length(Cells(processed_gene_data)) <= 2000) {
                if(length(unique(processed_gene_data$seurat_clusters)) > 1) {
                  processed_gene_data@assays$user_parameter_tracker = NULL
                  markers_genes = FindAllMarkers(processed_gene_data, assay = "RNA")
                  markers_genes = markers_genes %>% dplyr::filter(p_val_adj <= 0.05)
                  markers_genes = data.frame(markers_genes)
                  rm(processed_gene_data)
                } else {
                  markers_genes = NULL
                }
              } else {
                if(length(unique(processed_gene_data$seurat_clusters)) > 1) {
                  processed_gene_data@assays$user_parameter_tracker = NULL
                  processed_gene_data_sub = subset(processed_gene_data, downsample = 100)
                  rm(processed_gene_data)
                  Idents(processed_gene_data_sub) = "seurat_clusters"
                  markers_genes = FindAllMarkers(processed_gene_data_sub, assay = "RNA")
                  rm(processed_gene_data_sub)
                  markers_genes = markers_genes %>% dplyr::filter(p_val_adj <= 0.05)
                  markers_genes = data.frame(markers_genes)
                } else {
                  markers_genes = NULL
                }
              }
            } else {
              markers_genes = NULL
            }
            processed_isoform_data = isoform_seuratObj_processed()
            if(!is.null(processed_isoform_data)) {
              Idents(processed_isoform_data) = "seurat_clusters"
              if(length(Cells(processed_isoform_data)) <= 2000) {
                if(length(unique(processed_isoform_data$seurat_clusters)) > 1) {
                  markers_isoforms = FindAllMarkers(processed_isoform_data, assay = "RNA")
                  markers_isoforms = markers_isoforms %>% dplyr::filter(p_val_adj <= 0.05)
                  markers_isoforms = data.frame(markers_isoforms)
                  rm(processed_isoform_data)
                } else {
                  markers_isoforms = NULL
                } 
              } else {
                if(length(unique(processed_isoform_data$seurat_clusters)) > 1) {
                  processed_isoform_data_sub = subset(processed_isoform_data, downsample = 100)
                  rm(processed_isoform_data)
                  Idents(processed_isoform_data_sub) = "seurat_clusters"
                  markers_isoforms = FindAllMarkers(processed_isoform_data_sub, assay = "RNA")
                  rm(processed_isoform_data_sub)
                  markers_isoforms = markers_isoforms %>% dplyr::filter(p_val_adj <= 0.05)
                  markers_isoforms = data.frame(markers_isoforms)
                } else {
                  markers_isoforms = NULL
                }
              }
            } else {
              markers_isoforms = NULL
            }
          }
          if(input$runMarkers == "FALSE") {
            incProgress(0.4, detail = "markers calculation skipped ...")
            markers_genes = NULL
            markers_isoforms = NULL
          }
          processed_gene_data = gene_seuratObj_processed()
          processed_isoform_data = isoform_seuratObj_processed()
          parameter_list = processed_gene_data@assays$user_parameter_tracker
          check_original_presence = grep('Original Data: ', parameter_list, value = T)
          check_biotype_presence = grep('RNABiotype Data: ', parameter_list, value = T)
          check_varfeature_presence = grep('VariableFeatures Data: ', parameter_list, value = T)
          if(length(check_original_presence) > 0) {
            original_file_split = strsplit(check_original_presence, split = "Original Data: ", fixed = T)
            sc_original = readRDS(original_file_split[[1]][2])
            message('found original data')
          } else {
            sc_original = NULL
          }
          if(length(check_biotype_presence) > 0) {
            original_file_split = strsplit(check_biotype_presence, split = "RNABiotype Data: ", fixed = T)
            biotype_data_rna = read.table(original_file_split[[1]][2], sep = "\t", header = TRUE, check.names = FALSE)
            message('found biotype data')
          } else {
            biotype_data_rna = NULL
          }
          if(length(check_varfeature_presence) > 0) {
            original_file_split = strsplit(check_varfeature_presence, split = "VariableFeatures Data: ", fixed = T)
            varfeature_data_rna = read.table(original_file_split[[1]][2], sep = "\n", header = FALSE, check.names = FALSE)
            message('found var features for original data')
          } else {
            varfeature_data_rna = NULL
          }

        }, error = function(e) {
            shinyalert('Error in calculating markers', 'Please check your input parameters', type = "error", confirmButtonCol = "#1c82e0")
        })
        incProgress(0.2, detail = "markers calculation complete ...")
      })
      output$scRNA_Report <- downloadHandler(
        filename = function() {
          paste0("CogentDS_Analysis_Report", "-", Sys.Date(), ".html")
        },
        content = function(file) {
          withProgress(message = 'Preparing Report...', value = 0, {
            incProgress(0.2, detail = "Generating Report elements ...")
            tryCatch({
              incProgress(0.6, detail = "Generating report..please wait")
              Report_rmd <- paste0('R/Setup_Funtions/bin/', 'CogentDS_v2.0_scRNA_analyis_report_App.Rmd')
              params <- list(set_title = "Cogent™ NGS Discovery Software", seuratObject_gene_preprocessed = sc_original, seuratObject_gene_processed = processed_gene_data, seuratObject_isoform_processed = processed_isoform_data, biotype_data = biotype_data_rna, getMarkers = input$runMarkers, varfeauters_orig = varfeature_data_rna$V1, gene_markers = markers_genes, isoform_markers = markers_isoforms)
              rmarkdown::render(Report_rmd, output_file = file, params = params, envir = new.env(parent = globalenv()))
              incProgress(0.2, detail = "Complete")
              shinyalert('Analysis Report Downloaded', "Thank you! CogentDS Team", type = "success", confirmButtonCol = "#1c82e0")
            }, error = function(e) {
              shinyalert('Error in calculating markers', 'Please check your input parameters', type = "error", confirmButtonCol = "#1c82e0")
            })
          })
        }
      )
    })
  })
}
