######## CogentDS launcher ############
#source('R/installpackages.r')

source('R/landing_main_UI.R')

# Define UI
ui <- dashboardPage(
  skin = "purple",
  dashboardHeader(title = 'Cogent™ NGS Discovery Software', titleWidth = 450,
                            tags$li(a(href = 'https://www.takarabio.com/', img(src = 'logo.png', title = "Company Home", height = "30px"), style = "padding-top:10px; padding-bottom:10px;"), class = "dropdown"),
                            tags$li(class = "dropdown", actionButton("refreshapp", "Home", icon = icon("home"), style='color: black; background-color: white; border: transparent'))),
  dashboardSidebar(
    width = 150,
    sidebarMenuOutput("CogentDSsidebarmenu_main")
  ),

  dashboardBody(
    shinyjs::useShinyjs(),
    tags$head(tags$style(".sidebar-menu li { margin-bottom: 0px; margin-top: 0px;}")),
    tags$head(tags$style(".sidebar-menu li { padding: 0px 0px 0px 0px; display: block}")),
    tags$head(tags$style(HTML('

         .wrapper {height: auto !important; position:relative; overflow-x:hidden; overflow-y:hidden}

         .skin-purple .main-header .logo {
          background-color: white;
          color: black;
          border-bottom-style: solid;
          text-align: left;
          padding: 0px 0px 0px 10px;
          font-style: normal;
          font-family: math;
        }

        .skin-purple .main-header .navbar{
          background-color: white;
          border-bottom: #e4e4e4 solid;
        }

        .skin-purple .main-header .navbar .sidebar-toggle {
          background-color: white;
          color: black;
        }

        /* main sidebar */
          .skin-blue .main-sidebar {
            background-color: #e4e4e4;
        }

        .skin-purple .left-side, .skin-purple .main-sidebar, .skin-purple .wrapper {
          background-color: #e4e4e4;
          border-left-color: #e4e4e4;
        }

        .skin-purple .sidebar-menu>li.active>a, .skin-purple .sidebar-menu>li:hover>a {
          color: #1c82e0;
          text-align: left;
          background-color: #e4e4e4;
          border-left-color: #e4e4e4;
        }

        .skin-purple .main-header .navbar .sidebar-toggle:hover {
          background-color: white
        }

        .skin-purple .main-header .logo:hover {
          background-color: white
        }

        .skin-purple .sidebar a {
          color: darkgreen;
        }

        /* body */
          .content-wrapper, .right-side {
          background-color: white;
        }
        
    '))),
    tabItems(
      tabItem(tabName = "landing_main_UI", landing_main_UI("landing_main_UIModule"))
    )
  )
)

# Server Logic
server <- function(input, output, session) {
  
  ns <- session$ns

  ########## Load Server functions

  output$workflow <- renderImage({
  list(src = 'www/workflow.png',
         contentType = 'image/png',
         alt = "This is alternate text", height = "600")
  }, deleteFile = F)

  output$CogentDSsidebarmenu_main <- shinydashboard::renderMenu({
  sidebarMenu(id='CogentDSsidebarmenu_main',
              menuItem("CogentDSv2.1", tabName = "landing_main_UI", icon = icon("dashboard"), selected = "landing_main_UI"))

  })

  observeEvent(input$refreshapp, {
    shinyjs::refresh()
    output$CogentDSsidebarmenu_main <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu_main',
          menuItem("scRNA", tabName = "landing_main_UI", icon = icon("dashboard")))

    })
    updateTabItems(session, "CogentDSsidebarmenu_main", selected = "landing_main_UI")
  })

  observeEvent(input$scRNA, {
    
    withProgress(message = 'Starting Launch...', {
      incProgress(0.1, detail = 'Loading scRNA app...')
      incProgress(0.4, detail = 'Loading scRNA app, Launching...')
      rstudioapi::jobRunScript(path = 'bin/Launch_scRNA.app.R', workingDir = getwd(), name = "scRNA", importEnv = TRUE)
      Sys.sleep(25)
      incProgress(0.5, detail = 'Launched scRNA app')
      shinyalert("scRNA Application will be launched in a new browser tab", "Applications generally are launched within 30 seconds, but some machines may take longer", type = "info", confirmButtonCol = "#1c82e0")
    })
  })

 observeEvent(input$bulkRNA, {
 
    withProgress(message = 'Starting Launch...', {
      incProgress(0.1, detail = 'Loading bulkRNA app...')
      incProgress(0.4, detail = 'Loading bulkRNA app, Launching...')
      rstudioapi::jobRunScript(path = 'bin/Launch_bulkRNA.app.R', workingDir = getwd(), name = "bulkRNA", importEnv = TRUE)
      Sys.sleep(25)
      incProgress(0.5, detail = 'Launched bulkRNA app')
      shinyalert("Bulk RNA Application will be launched in a new browser tab", "Applications generally are launched within 30 seconds, but some machines may take longer", type = "info", confirmButtonCol = "#1c82e0")
    })
  })

  observeEvent(input$scDNA, {
 
    withProgress(message = 'Starting Launch...', {
      incProgress(0.1, detail = 'Loading scDNA app...')
      incProgress(0.4, detail = 'Loading scDNA app, Launching...')
      rstudioapi::jobRunScript(path = 'bin/Launch_scDNA.app.R', workingDir = getwd(), name = "scDNA", importEnv = TRUE)
      Sys.sleep(25)
      incProgress(0.5, detail = 'Launched scDNA app')
      shinyalert("scDNA Application will be launched in a new browser tab", "Applications generally are launched within 30 seconds, but some machines may take longer", type = "info", confirmButtonCol = "#1c82e0")
    })
  })
}

shinyApp(ui = ui, server = server)
